@@ -33,10 +33,16 @@
 #  define VTKSLACFILTERS_DEPRECATED_NO_EXPORT VTKSLACFILTERS_NO_EXPORT VTKSLACFILTERS_DEPRECATED
 #endif
 
+/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
 #if 0 /* DEFINE_NO_DEPRECATED */
 #  ifndef VTKSLACFILTERS_NO_DEPRECATED
 #    define VTKSLACFILTERS_NO_DEPRECATED
 #  endif
 #endif
 
+/* VTK-HeaderTest-Exclude: vtkSLACFiltersModule.h */
+
+/* Include ABI Namespace */
+#include "vtkABINamespace.h"
+
 #endif /* VTKSLACFILTERS_EXPORT_H */
