@@ -33,10 +33,16 @@
 #  define VTKREMOTINGCORE_DEPRECATED_NO_EXPORT VTKREMOTINGCORE_NO_EXPORT VTKREMOTINGCORE_DEPRECATED
 #endif
 
+/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
 #if 0 /* DEFINE_NO_DEPRECATED */
 #  ifndef VTKREMOTINGCORE_NO_DEPRECATED
 #    define VTKREMOTINGCORE_NO_DEPRECATED
 #  endif
 #endif
 
+/* VTK-HeaderTest-Exclude: vtkRemotingCoreModule.h */
+
+/* Include ABI Namespace */
+#include "vtkABINamespace.h"
+
 #endif /* VTKREMOTINGCORE_EXPORT_H */
