@@ -5,18 +5,15 @@ pipeline. Additionally, this module has several other utility functions that are
 appropriate for co-processing.
 """
 
-# for Python2 print statmements to output like Python3 print statements
-from __future__ import print_function
 from paraview import simple, servermanager
 from paraview.detail import exportnow
 
-import math
-
 # If the user created a filename in a location that doesn't exist by default we'll
 # make the directory for them. This can be changed though by setting createDirectoriesIfNeeded
 # to False.
 createDirectoriesIfNeeded = True
 
+
 # -----------------------------------------------------------------------------
 
 class CoProcessor(object):
@@ -62,12 +59,13 @@ class CoProcessor(object):
         self.__CinemaTracks = {}
         self.__InitialFrequencies = {}
         self.__PrintEnsightFormatString = False
-        self.__TimeStepToStartOutputAt=0
-        self.__ForceOutputAtFirstCall=False
+        self.__TimeStepToStartOutputAt = 0
+        self.__ForceOutputAtFirstCall = False
         self.__FirstTimeStepIndex = None
         # a list of arrays requested for each channel, e.g. {'input': ["a point data array name", 0], ["a cell data array name", 1]}
         self.__RequestedArrays = None
-        self.__RootDirectory = ""
+        self.__ImageRootDirectory = ""
+        self.__DataRootDirectory = ""
         self.__CinemaDHelper = None
 
     def SetPrintEnsightFormatString(self, enable):
@@ -84,8 +82,8 @@ class CoProcessor(object):
            input, and value is a list of frequencies.
            """
         if type(frequencies) != dict:
-           raise RuntimeError (
-                 "Incorrect argument type: %s, must be a dict" % type(frequencies))
+            raise RuntimeError(
+                "Incorrect argument type: %s, must be a dict" % type(frequencies))
         self.__InitialFrequencies = frequencies
 
     def SetRequestedArrays(self, channelname, requestedarrays):
@@ -103,10 +101,10 @@ class CoProcessor(object):
            input, and value is a list of frequencies.
            """
 
-        self.__TimeStepToStartOutputAt=timeStepToStartOutputAt
-        self.__ForceOutputAtFirstCall=forceOutputAtFirstCall
+        self.__TimeStepToStartOutputAt = timeStepToStartOutputAt
+        self.__ForceOutputAtFirstCall = forceOutputAtFirstCall
 
-    def EnableLiveVisualization(self, enable, frequency = 1):
+    def EnableLiveVisualization(self, enable, frequency=1):
         """Call this method to enable live-visualization. When enabled,
         DoLiveVisualization() will communicate with ParaView server if possible
         for live visualization. Frequency specifies how often the
@@ -117,7 +115,7 @@ class CoProcessor(object):
     def CreatePipeline(self, datadescription):
         """This methods must be overridden by subclasses to create the
            visualization pipeline."""
-        raise RuntimeError ("Subclasses must override this method.")
+        raise RuntimeError("Subclasses must override this method.")
 
     def LoadRequestedData(self, datadescription):
         """Call this method in RequestDataDescription co-processing pass to mark
@@ -136,23 +134,29 @@ class CoProcessor(object):
         # generating the script then only those arrays will be provided to the Live connection.
         # note that we want the pipeline built before we do the actual first live connection.
         if self.__EnableLiveVisualization and self.NeedToOutput(datadescription, self.__LiveVisualizationFrequency) \
-           and self.__LiveVisualizationLink:
+                and self.__LiveVisualizationLink:
             if self.__LiveVisualizationLink.Initialize(servermanager.ActiveConnection.Session.GetSessionProxyManager()):
                 if self.__RequestedArrays:
                     for key in self.__RequestedArrays:
                         for v in self.__RequestedArrays[key]:
-                            datadescription.GetInputDescriptionByName(key).AddField(v[0], v[1])
+                            inputDescription = datadescription.GetInputDescriptionByName(key)
+                            if inputDescription:
+                                inputDescription.AddField(v[0], v[1])
                 elif self.__InitialFrequencies:
                     # __ProducersMap will not be filled up until after the first call to
                     # DoCoProcessing so we rely on __InitialFrequencies initially but then
                     # __ProducersMap after that as __InitialFrequencies will be cleared out.
                     for key in self.__InitialFrequencies:
-                        datadescription.GetInputDescriptionByName(key).AllFieldsOn()
-                        datadescription.GetInputDescriptionByName(key).GenerateMeshOn()
+                        inputDescription = datadescription.GetInputDescriptionByName(key)
+                        if inputDescription:
+                            inputDescription.AllFieldsOn()
+                            inputDescription.GenerateMeshOn()
                 else:
                     for key in self.__ProducersMap:
-                        datadescription.GetInputDescriptionByName(key).AllFieldsOn()
-                        datadescription.GetInputDescriptionByName(key).GenerateMeshOn()
+                        inputDescription = datadescription.GetInputDescriptionByName(key)
+                        if inputDescription:
+                            inputDescription.AllFieldsOn()
+                            inputDescription.GenerateMeshOn()
                 return
 
         # if we haven't processed the pipeline yet in DoCoProcessing() we
@@ -164,13 +168,17 @@ class CoProcessor(object):
             if self.__RequestedArrays:
                 for key in self.__RequestedArrays:
                     for v in self.__RequestedArrays[key]:
-                        datadescription.GetInputDescriptionByName(key).AddField(v[0], v[1])
+                        inputDescription = datadescription.GetInputDescriptionByName(key)
+                        if inputDescription:
+                            inputDescription.AddField(v[0], v[1])
             elif self.__InitialFrequencies:
                 for key in self.__InitialFrequencies:
                     freqs = self.__InitialFrequencies.get(key, [])
                     if self.__EnableLiveVisualization or self.IsInModulo(datadescription, freqs):
-                        datadescription.GetInputDescriptionByName(key).AllFieldsOn()
-                        datadescription.GetInputDescriptionByName(key).GenerateMeshOn()
+                        inputDescription = datadescription.GetInputDescriptionByName(key)
+                        if inputDescription:
+                            inputDescription.AllFieldsOn()
+                            inputDescription.GenerateMeshOn()
         else:
             # the catalyst pipeline may have been changed by a Live connection
             # so we need to regenerate the frequencies
@@ -191,7 +199,7 @@ class CoProcessor(object):
 
             for view in self.__ViewsList:
                 if (view.cpFrequency and self.NeedToOutput(datadescription, view.cpFrequency)) or \
-                   datadescription.GetForceOutput() == True:
+                        datadescription.GetForceOutput() == True:
                     viewinputs = cpstate.locate_simulation_inputs_for_view(view)
                     for viewinput in viewinputs:
                         if self.__RequestedArrays:
@@ -202,19 +210,18 @@ class CoProcessor(object):
                             datadescription.GetInputDescriptionByName(viewinput).AllFieldsOn()
                             datadescription.GetInputDescriptionByName(viewinput).GenerateMeshOn()
 
-
     def UpdateProducers(self, datadescription):
         """This method will update the producers in the pipeline. If the
            pipeline is not created, it will be created using
            self.CreatePipeline().
         """
         if not self.__PipelineCreated:
-           self.CreatePipeline(datadescription)
-           self.__PipelineCreated = True
-           if self.__EnableLiveVisualization:
-               # we don't want to use __InitialFrequencies any more with live viz
-               self.__InitialFrequencies = None
-           self.__FixupWriters()
+            self.CreatePipeline(datadescription)
+            self.__PipelineCreated = True
+            if self.__EnableLiveVisualization:
+                # we don't want to use __InitialFrequencies any more with live viz
+                self.__InitialFrequencies = None
+            self.__FixupWriters()
 
         else:
             simtime = datadescription.GetTime()
@@ -223,7 +230,6 @@ class CoProcessor(object):
                     datadescription.GetInputDescriptionByName(name).GetGrid(),
                     simtime)
 
-
     def WriteData(self, datadescription):
         """This method will update all writes present in the pipeline, as
            needed, to generate the output data files, respecting the
@@ -237,7 +243,7 @@ class CoProcessor(object):
                 paddingamount = writer.parameters.GetProperty("PaddingAmount").GetElement(0)
                 helperName = writer.GetXMLName()
                 if helperName == "ExodusIIWriter":
-                    ts = "."+str(timestep).rjust(paddingamount, '0')
+                    ts = "." + str(timestep).rjust(paddingamount, '0')
                     writer.FileName = fileName + ts
                 else:
                     ts = str(timestep).rjust(paddingamount, '0')
@@ -253,7 +259,7 @@ class CoProcessor(object):
                             os.makedirs(newDir)
                         except OSError:
                             if not os.path.isdir(newDir):
-                                print ("ERROR: Cannot make directory for", writer.FileName, ". No data will be written.")
+                                print("ERROR: Cannot make directory for", writer.FileName, ". No data will be written.")
                                 oktowrite[0] = 0.
                     comm.Broadcast(oktowrite, 1, 0)
                     if oktowrite[0] == 0:
@@ -263,7 +269,6 @@ class CoProcessor(object):
                 self.__AppendToCinemaDTable(timestep, "writer_%s" % self.__WritersList.index(writer), writer.FileName)
         self.__FinalizeCinemaDTable()
 
-
     def WriteImages(self, datadescription, rescale_lookuptable=False,
                     image_quality=None, padding_amount=0):
         """This method will update all views, if present and write output
@@ -300,7 +305,7 @@ class CoProcessor(object):
         cinema_dirs = []
         for view in self.__ViewsList:
             if (view.cpFrequency and self.NeedToOutput(datadescription, view.cpFrequency)) or \
-               datadescription.GetForceOutput() == True:
+                    datadescription.GetForceOutput() == True:
                 fname = view.cpFileName
                 ts = str(timestep).rjust(padding_amount, '0')
                 fname = fname.replace("%t", ts)
@@ -311,7 +316,7 @@ class CoProcessor(object):
                     elif view.IsA("vtkSMContextViewProxy") == True:
                         view.ResetDisplay()
                     else:
-                        print (' do not know what to do with a ', view.GetClassName())
+                        print(' do not know what to do with a ', view.GetClassName())
                 view.ViewTime = datadescription.GetTime()
                 if rescale_lookuptable:
                     self.RescaleDataRange(view, datadescription.GetTime())
@@ -319,10 +324,10 @@ class CoProcessor(object):
                 if cinemaOptions and 'camera' in cinemaOptions:
                     if 'composite' in view.cpCinemaOptions and view.cpCinemaOptions['composite'] == True:
                         dirname, filelist = self.UpdateCinema(view, datadescription,
-                                                    specLevel="B")
+                                                              specLevel="B")
                     else:
                         dirname, filelist = self.UpdateCinema(view, datadescription,
-                                                    specLevel="A")
+                                                              specLevel="A")
                     if dirname:
                         self.__AppendCViewToCinemaDTable(timestep, "view_%s" % self.__ViewsList.index(view), filelist)
                         cinema_dirs.append(dirname)
@@ -338,7 +343,7 @@ class CoProcessor(object):
                                 os.makedirs(newDir)
                             except OSError:
                                 if not os.path.isdir(newDir):
-                                    print ("ERROR: Cannot make directory for", fname, ". No image will be output.")
+                                    print("ERROR: Cannot make directory for", fname, ". No image will be output.")
                                     oktowrite[0] = 0.
                         comm.Broadcast(oktowrite, 1, 0)
                         if oktowrite[0] == 0:
@@ -355,7 +360,7 @@ class CoProcessor(object):
                         # let simple.SaveScreenshot pick a default.
                         quality = None
 
-                    if fname.endswith('png') and view.cpCompression is not None and view.cpCompression != -1 :
+                    if fname.endswith('png') and view.cpCompression is not None and view.cpCompression != -1:
                         simple.SaveScreenshot(fname, view,
                                               CompressionLevel=view.cpCompression,
                                               ImageResolution=view.ViewSize)
@@ -371,7 +376,6 @@ class CoProcessor(object):
 
         self.__FinalizeCinemaDTable()
 
-
     def DoLiveVisualization(self, datadescription, hostname, port):
         """This method execute the code-stub needed to communicate with ParaView
            for live-visualization. Call this method only if you want to support
@@ -392,9 +396,9 @@ class CoProcessor(object):
             # Initialize the "link"
             self.__LiveVisualizationLink.Initialize(servermanager.ActiveConnection.Session.GetSessionProxyManager())
 
-
         if self.__EnableLiveVisualization and self.NeedToOutput(datadescription, self.__LiveVisualizationFrequency):
-            if not self.__LiveVisualizationLink.Initialize(servermanager.ActiveConnection.Session.GetSessionProxyManager()):
+            if not self.__LiveVisualizationLink.Initialize(
+                    servermanager.ActiveConnection.Session.GetSessionProxyManager()):
                 return
 
         time = datadescription.GetTime()
@@ -421,7 +425,7 @@ class CoProcessor(object):
                 # and then it continues the loop. Returns != 0 if LIVE side
                 # disconnects
                 if (self.__LiveVisualizationLink.WaitForLiveChange()):
-                    break;
+                    break
             else:
                 break
 
@@ -431,13 +435,16 @@ class CoProcessor(object):
         # Check that the producer name for the input given is valid for the
         # current setup.
         if not datadescription.GetInputDescriptionByName(inputname):
-            raise RuntimeError ("Simulation input name '%s' does not exist" % inputname)
+            raise RuntimeError("Simulation input name '%s' does not exist" % inputname)
 
         grid = datadescription.GetInputDescriptionByName(inputname).GetGrid()
         if not grid:
             # we have a description of this channel but we don't need the grid so return
             return
 
+        if inputname in self.__ProducersMap:
+            raise RuntimeError("CreateProducer is being called multiple times for input '%s'" % inputname)
+
         producer = simple.PVTrivialProducer(guiName=inputname)
         producer.add_attribute("cpSimulationInput", inputname)
         # mark this as an input proxy so we can use cpstate.locate_simulation_inputs()
@@ -452,13 +459,29 @@ class CoProcessor(object):
                 grid.IsA("vtkStructuredGrid") == True or \
                 grid.IsA("vtkRectilinearGrid") == True:
             extent = datadescription.GetInputDescriptionByName(inputname).GetWholeExtent()
-            producer.WholeExtent= [ extent[0], extent[1], extent[2], extent[3], extent[4], extent[5] ]
+            producer.WholeExtent = [extent[0], extent[1], extent[2], extent[3], extent[4], extent[5]]
 
         # Save the producer for easy access in UpdateProducers() call.
         self.__ProducersMap[inputname] = producer
         producer.UpdatePipeline(datadescription.GetTime())
         return producer
 
+    def CreateTemporalProducer(self, datadescription, inputname):
+        """Python access to a temporal cache object associated with a specific
+        one simulation product. Much like CreateProducer, only this ends up with
+        a temporal cache filter instead of a PVTrivialProducer."""
+        if not datadescription.GetInputDescriptionByName(inputname):
+            raise RuntimeError("Simulation input name '%s' does not exist" % inputname)
+
+        idd = datadescription.GetInputDescriptionByName(inputname)
+
+        cache = idd.GetTemporalCache()
+        if not cache:
+            raise RuntimeError("I see no cache for '%s'" % inputname)
+            return
+
+        return servermanager._getPyProxy(cache)
+
     def ProcessExodusIIWriter(self, writer):
         """Extra work for the ExodusII writer to avoid undesired warnings
            and print out a message on how to read the files into Ensight."""
@@ -475,13 +498,15 @@ class CoProcessor(object):
                 if nump == 1:
                     print("Ensight 'Set string' input is '", writer.FileName, ".*'", sep="")
                 else:
-                    print("Ensight 'Set string' input is '", writer.FileName, ".*."+str(nump)+ \
-                          ".<"+str(nump)+":%0."+str(len(str(nump-1)))+"d>'", sep="")
+                    print("Ensight 'Set string' input is '", writer.FileName, ".*." + str(nump) + \
+                          ".<" + str(nump) + ":%0." + str(len(str(nump - 1))) + "d>'", sep="")
 
     def RegisterWriter(self, writer, filename, freq, paddingamount=0, **params):
         """Registers a writer proxy. This method is generally used in
            CreatePipeline() to register writers. All writes created as such will
-           write the output files appropriately in WriteData() is called."""
+           write the output files appropriately in WriteData() is called.
+           params should be empty as of ParaView 5.9 but is passed in for
+           backwards compatibility."""
         writerParametersProxy = self.WriterParametersProxy(
             writer, filename, freq, paddingamount)
 
@@ -493,7 +518,6 @@ class CoProcessor(object):
                 wp = writer.GetProperty(p)
                 wp.SetData(v)
 
-
         self.__WritersList.append(writer)
 
         helperName = writer.GetXMLName()
@@ -519,6 +543,7 @@ class CoProcessor(object):
         proxy = servermanager.ProxyManager().NewProxy(
             "insitu_writer_parameters", helperName)
         controller.PreInitializeProxy(proxy)
+
         if writerIsProxy:
             # it's possible that the writer can take in multiple input connections
             # so we need to go through all of them. the try/except block seems
@@ -544,8 +569,8 @@ class CoProcessor(object):
         Register a point of control (filter's property) that will be varied over in a cinema export.
         """
         if not isinstance(proxy, servermanager.Proxy):
-            raise RuntimeError ("Invalid 'proxy' argument passed to RegisterCinemaTrack.")
-        self.__CinemaTracksList.append({"name":name, "proxy":proxy, "smproperty":smproperty, "valrange":valrange})
+            raise RuntimeError("Invalid 'proxy' argument passed to RegisterCinemaTrack.")
+        self.__CinemaTracksList.append({"name": name, "proxy": proxy, "smproperty": smproperty, "valrange": valrange})
         proxyDefinitions = self.__CinemaTracks[proxy] if (proxy in self.__CinemaTracks) else {}
         proxyDefinitions[smproperty] = valrange
         self.__CinemaTracks[proxy] = proxyDefinitions
@@ -554,7 +579,7 @@ class CoProcessor(object):
     def AddArraysToCinemaTrack(self, proxy, propertyName, arrayNames):
         ''' Register user-defined target arrays by name. '''
         if not isinstance(proxy, servermanager.Proxy):
-            raise RuntimeError ("Invalid 'proxy' argument passed to AddArraysToCinemaTrack.")
+            raise RuntimeError("Invalid 'proxy' argument passed to AddArraysToCinemaTrack.")
 
         proxyDefinitions = self.__CinemaTracks[proxy] if (proxy in self.__CinemaTracks) else {}
         proxyDefinitions[propertyName] = arrayNames
@@ -566,34 +591,17 @@ class CoProcessor(object):
         """Register a view for image capture with extra meta-data such
         as magnification, size and frequency."""
         if not isinstance(view, servermanager.Proxy):
-            raise RuntimeError ("Invalid 'view' argument passed to RegisterView.")
+            raise RuntimeError("Invalid 'view' argument passed to RegisterView.")
         view.add_attribute("cpFileName", filename)
         view.add_attribute("cpFrequency", freq)
         view.add_attribute("cpFitToScreen", fittoscreen)
         view.add_attribute("cpMagnification", magnification)
         view.add_attribute("cpCinemaOptions", cinema)
         view.add_attribute("cpCompression", compression)
-        view.ViewSize = [ width, height ]
+        view.ViewSize = [width, height]
         self.__ViewsList.append(view)
         return view
 
-    def CreateWriter(self, proxy_ctor, filename, freq):
-        """**DEPRECATED!!! Use RegisterWriter instead**
-        Creates a writer proxy. This method is generally used in
-        reatePipeline() to create writers. All writes created as such will
-        write the output files appropriately in WriteData() is called.
-        """
-        writer = proxy_ctor()
-        return self.RegisterWriter(writer, filename, freq)
-
-    def CreateView(self, proxy_ctor, filename, freq, fittoscreen, magnification, width, height):
-        """**DEPRECATED!!! Use RegisterView instead**
-        Create a CoProcessing view for image capture with extra meta-data
-        such as magnification, size and frequency.
-        """
-        view = proxy_ctor()
-        return self.RegisterView(view, filename, freq, fittoscreen, magnification, width, height, None)
-
     def Finalize(self):
         for writer in self.__WritersList:
             if hasattr(writer, 'Finalize'):
@@ -608,15 +616,15 @@ class CoProcessor(object):
         reps = view.Representations
         for rep in reps:
             if not hasattr(rep, 'Visibility') or \
-                not rep.Visibility or \
-                not hasattr(rep, 'MapScalars') or \
-                not rep.MapScalars or \
-                not rep.LookupTable:
+                    not rep.Visibility or \
+                    not hasattr(rep, 'MapScalars') or \
+                    not rep.MapScalars or \
+                    not rep.LookupTable:
                 # rep is either not visible or not mapping scalars using a LUT.
                 continue;
 
             input = rep.Input
-            input.UpdatePipeline(time) #make sure range is up-to-date
+            input.UpdatePipeline(time)  # make sure range is up-to-date
             lut = rep.LookupTable
 
             colorArrayInfo = rep.GetArrayInformationForColorArray()
@@ -625,7 +633,7 @@ class CoProcessor(object):
                 datarange = [sys.float_info.max, -sys.float_info.max]
             else:
                 if lut.VectorMode != 'Magnitude' or \
-                   colorArrayInfo.GetNumberOfComponents() == 1:
+                        colorArrayInfo.GetNumberOfComponents() == 1:
                     datarange = colorArrayInfo.GetComponentRange(lut.VectorComponent)
                 else:
                     # -1 corresponds to the magnitude.
@@ -637,33 +645,35 @@ class CoProcessor(object):
             globalController = pm.GetGlobalController()
             localarray = vtkDoubleArray()
             localarray.SetNumberOfTuples(2)
-            localarray.SetValue(0, -datarange[0]) # negate so that MPI_MAX gets min instead of doing a MPI_MIN and MPI_MAX
+            localarray.SetValue(0,
+                                -datarange[0])  # negate so that MPI_MAX gets min instead of doing a MPI_MIN and MPI_MAX
             localarray.SetValue(1, datarange[1])
             globalarray = vtkDoubleArray()
             globalarray.SetNumberOfTuples(2)
             globalController.AllReduce(localarray, globalarray, 0)
             globaldatarange = [-globalarray.GetValue(0), globalarray.GetValue(1)]
             rgbpoints = lut.RGBPoints.GetData()
-            numpts = len(rgbpoints)//4
-            if globaldatarange[0] != rgbpoints[0] or globaldatarange[1] != rgbpoints[(numpts-1)*4]:
+            numpts = len(rgbpoints) // 4
+            if globaldatarange[0] != rgbpoints[0] or globaldatarange[1] != rgbpoints[(numpts - 1) * 4]:
                 # rescale all of the points
-                oldrange = rgbpoints[(numpts-1)*4] - rgbpoints[0]
+                oldrange = rgbpoints[(numpts - 1) * 4] - rgbpoints[0]
                 newrange = globaldatarange[1] - globaldatarange[0]
                 # only readjust if the new range isn't zero.
                 if newrange != 0:
-                   newrgbpoints = list(rgbpoints)
-                   # if the old range isn't 0 then we use that ranges distribution
-                   if oldrange != 0:
-                      for v in range(numpts-1):
-                         newrgbpoints[v*4] = globaldatarange[0]+(rgbpoints[v*4] - rgbpoints[0])*newrange/oldrange
+                    newrgbpoints = list(rgbpoints)
+                    # if the old range isn't 0 then we use that ranges distribution
+                    if oldrange != 0:
+                        for v in range(numpts - 1):
+                            newrgbpoints[v * 4] = globaldatarange[0] + (
+                                        rgbpoints[v * 4] - rgbpoints[0]) * newrange / oldrange
 
-                      # avoid numerical round-off, at least with the last point
-                      newrgbpoints[(numpts-1)*4] = globaldatarange[1]
-                   else: # the old range is 0 so the best we can do is to space the new points evenly
-                      for v in range(numpts+1):
-                         newrgbpoints[v*4] = globaldatarange[0]+v*newrange/(1.0*numpts)
+                        # avoid numerical round-off, at least with the last point
+                        newrgbpoints[(numpts - 1) * 4] = globaldatarange[1]
+                    else:  # the old range is 0 so the best we can do is to space the new points evenly
+                        for v in range(numpts + 1):
+                            newrgbpoints[v * 4] = globaldatarange[0] + v * newrange / (1.0 * numpts)
 
-                   lut.RGBPoints.SetData(newrgbpoints)
+                    lut.RGBPoints.SetData(newrgbpoints)
 
     def UpdateCinema(self, view, datadescription, specLevel):
         """ called from catalyst at each timestep to add to the cinema database """
@@ -680,29 +690,29 @@ class CoProcessor(object):
             paraview.print_error(e)
             return
 
-        #figure out where to put this store
+        # figure out where to put this store
         import os.path
         vfname = view.cpFileName
         extension = os.path.splitext(vfname)[1]
-        vfname = vfname[0:vfname.rfind("_")] #strip _num.ext
+        vfname = vfname[0:vfname.rfind("_")]  # strip _num.ext
         fname = os.path.join(os.path.dirname(vfname),
                              "cinema",
                              os.path.basename(vfname),
                              "info.json")
 
         def float_limiter(x):
-            #a shame, but needed to make sure python, javascript and (directory/file)name agree
+            # a shame, but needed to make sure python, javascript and (directory/file)name agree
             if isinstance(x, (float)):
-                return '%.6e' % x #arbitrarily chose 6 significant digits
+                return '%.6e' % x  # arbitrarily chose 6 significant digits
             else:
                 return x
 
-        #what time?
+        # what time?
         time = datadescription.GetTime()
         view.ViewTime = time
         formatted_time = float_limiter(time)
 
-        #ensure that cinema operates on the specified view
+        # ensure that cinema operates on the specified view
         simple.SetActiveView(view)
 
         # Include camera information in the user defined parameters.
@@ -720,51 +730,51 @@ class CoProcessor(object):
         if "tracking" in co:
             tracking_def = co['tracking']
 
-        #figure out what we show now
-        pxystate= pv_introspect.record_visibility()
+        # figure out what we show now
+        pxystate = pv_introspect.record_visibility()
         # a conservative global bounds for consistent z scaling
-        minbds, maxbds  = pv_introspect.max_bounds()
+        minbds, maxbds = pv_introspect.max_bounds()
 
-        #make sure depth rasters are consistent
+        # make sure depth rasters are consistent
         view.MaxClipBounds = [minbds, maxbds, minbds, maxbds, minbds, maxbds]
         view.LockBounds = 1
 
         disableValues = False if 'noValues' not in co else co['noValues']
 
-        if specLevel=="B":
+        if specLevel == "B":
             p = pv_introspect.inspect(skip_invisible=True)
         else:
             p = pv_introspect.inspect(skip_invisible=False)
         fs = pv_introspect.make_cinema_store(p, fname, view,
-                                             forcetime = formatted_time,
-                                             userDefined = self.__CinemaTracks,
-                                             specLevel = specLevel,
-                                             camType = camType,
-                                             extension = extension,
-                                             disableValues = disableValues)
+                                             forcetime=formatted_time,
+                                             userDefined=self.__CinemaTracks,
+                                             specLevel=specLevel,
+                                             camType=camType,
+                                             extension=extension,
+                                             disableValues=disableValues)
 
-        #all nodes participate, but only root can writes out the files
+        # all nodes participate, but only root can writes out the files
         pm = servermanager.vtkProcessModule.GetProcessModule()
         pid = pm.GetPartitionId()
 
         enableFloatVal = False if 'floatValues' not in co else co['floatValues']
 
         new_files = {}
-        ret = pv_introspect.explore(fs, p, iSave = (pid == 0),
-                              currentTime = {'time':formatted_time},
-                              userDefined = self.__CinemaTracks,
-                              specLevel = specLevel,
-                              camType = camType,
-                              tracking = tracking_def,
-                              floatValues = enableFloatVal,
-                              disableValues = disableValues)
+        ret = pv_introspect.explore(fs, p, iSave=(pid == 0),
+                                    currentTime={'time': formatted_time},
+                                    userDefined=self.__CinemaTracks,
+                                    specLevel=specLevel,
+                                    camType=camType,
+                                    tracking=tracking_def,
+                                    floatValues=enableFloatVal,
+                                    disableValues=disableValues)
         if pid == 0:
             fs.save()
         new_files[vfname] = ret;
 
         view.LockBounds = 0
 
-        #restore what we showed
+        # restore what we showed
         pv_introspect.restore_visibility(pxystate)
         return os.path.basename(vfname), new_files
 
@@ -773,11 +783,11 @@ class CoProcessor(object):
         Return True if the given timestep in datadescription is in one of the provided frequencies
         or output is forced. This can be interpreted as follow::
 
-        isFM = IsInModulo(timestep-timeStepToStartOutputAt, [2,3,7])
+            isFM = IsInModulo(timestep-timeStepToStartOutputAt, [2,3,7])
 
         is similar to::
 
-        isFM = (timestep-timeStepToStartOutputAt % 2 == 0) or (timestep-timeStepToStartOutputAt % 3 == 0) or (timestep-timeStepToStartOutputAt % 7 == 0)
+            isFM = (timestep-timeStepToStartOutputAt % 2 == 0) or (timestep-timeStepToStartOutputAt % 3 == 0) or (timestep-timeStepToStartOutputAt % 7 == 0)
 
         The timeStepToStartOutputAt is the first timestep that will potentially be output.
         """
@@ -790,7 +800,6 @@ class CoProcessor(object):
 
         return False
 
-
     def NeedToOutput(self, datadescription, frequency):
         """
         Return True if we need to output based on the input timestep, frequency and forceOutput. Checks based
@@ -807,16 +816,15 @@ class CoProcessor(object):
         if self.__ForceOutputAtFirstCall and self.__FirstTimeStepIndex == timestep:
             return True
 
-        if self.__TimeStepToStartOutputAt <= timestep and (timestep-self.__TimeStepToStartOutputAt) % frequency == 0:
+        if self.__TimeStepToStartOutputAt <= timestep and (timestep - self.__TimeStepToStartOutputAt) % frequency == 0:
             return True
 
         return False
 
-
     def EnableCinemaDTable(self):
         """ Enable the normally disabled cinema D table export feature """
-        self.__CinemaDHelper = exportnow.CinemaDHelper(True, self.__RootDirectory)
-
+        self.__CinemaDHelper = exportnow.CinemaDHelper(True,
+                                                       self.__ImageRootDirectory)
 
     def __AppendCViewToCinemaDTable(self, time, producer, filelist):
         """
@@ -831,7 +839,6 @@ class CoProcessor(object):
         if comm.GetLocalProcessId() == 0:
             self.__CinemaDHelper.AppendCViewToCinemaDTable(time, producer, filelist)
 
-
     def __AppendToCinemaDTable(self, time, producer, filename):
         """
         This is called every time catalyst writes any data file or screenshot
@@ -853,21 +860,30 @@ class CoProcessor(object):
         if comm.GetLocalProcessId() == 0:
             self.__CinemaDHelper.WriteNow()
 
-
     def SetRootDirectory(self, root_directory):
         """ Makes Catalyst put all output under this directory. """
+        self.SetImageRootDirectory(root_directory)
+        self.SetDataRootDirectory(root_directory)
+
+    def SetImageRootDirectory(self, root_directory):
+        """Specify root directory for image extracts"""
         if root_directory and not root_directory.endswith("/"):
             root_directory = root_directory + "/"
-        self.__RootDirectory = root_directory
+        self.__ImageRootDirectory = root_directory
 
+    def SetDataRootDirectory(self, root_directory):
+        """Specify root directory for data extracts"""
+        if root_directory and not root_directory.endswith("/"):
+            root_directory = root_directory + "/"
+        self.__DataRootDirectory = root_directory
 
     def __FixupWriters(self):
         """ Called once to ensure that all writers obey the root directory directive """
-        if not self.__RootDirectory:
-            return
-        for view in self.__ViewsList:
-            view.cpFileName = self.__RootDirectory + view.cpFileName
-        for writer in self.__WritersList:
-            fileName = self.__RootDirectory + writer.parameters.GetProperty("FileName").GetElement(0)
-            writer.parameters.GetProperty("FileName").SetElement(0, fileName)
-            writer.parameters.FileName = fileName
+        if self.__ImageRootDirectory:
+            for view in self.__ViewsList:
+                view.cpFileName = self.__ImageRootDirectory + view.cpFileName
+        if self.__DataRootDirectory:
+            for writer in self.__WritersList:
+                fileName = self.__DataRootDirectory + writer.parameters.GetProperty("FileName").GetElement(0)
+                writer.parameters.GetProperty("FileName").SetElement(0, fileName)
+                writer.parameters.FileName = fileName
